from typing import Optional

from astris.lib import Div, Svg, Use, H3, P, A


class FeatureList(Div):
    def __init__(self, features: list[dict]) -> None:
        super().__init__(
            class_name="row g-4 py-5 row-cols-1 row-cols-lg-3",
            children=[
                IconFeature(
                    icon_class=feature.get("icon_class", ""),
                    title=feature.get("title", ""),
                    description=feature.get("description", ""),
                    call_to_action=[
                        A(
                            class_name="icon-link",
                            children=["Learn More"],
                            href=feature.get("href", "#"),
                        )
                    ],
                )
                for feature in features
            ],
        )


class IconFeature(Div):
    def __init__(
        self,
        icon_class: str,
        title: str,
        description: str,
        call_to_action: Optional[list] = None,
    ) -> None:

        children = [
            Div(
                class_name="feature-icon d-inline-flex align-items-center justify-content-center text-bg-primary bg-gradient fs-2 mb-3",
                children=[
                    Svg(
                        class_name="bi",
                        width="1em",
                        height="1em",
                        aria_hidden="true",
                        children=[Use(xlink_href=f"#{icon_class}")],
                    )
                ],
            ),
            H3(class_name="fs-2 text-body-emphasis", children=[title]),
            P(children=[description]),
        ]

        if call_to_action is not None:
            children.extend(call_to_action)

        super().__init__(
            class_name="feature col",
            children=children,
        )



from astris.layout import Column, Container, Row


def test_layout_wrappers_render_div_tag() -> None:
    container = Container(children=["content"])
    column = Column(children=["content"])
    row = Row(children=["content"])

    assert container.render() == "<div>content</div>"
    assert (
        column.render()
        == '<div style="display: flex; flex-direction: column;">content</div>'
    )
    assert (
        row.render() == '<div style="display: flex; flex-direction: row;">content</div>'
    )



from .app import Astris
from .component import Component, Element, Text
from .content import JsonCollection, JsonCollectionEntry, register_json_collection

__version__ = "0.1.4"

__all__ = [
    "Astris",
    "Component",
    "Element",
    "Text",
    "JsonCollection",
    "JsonCollectionEntry",
    "register_json_collection",
    "__version__",
]



from __future__ import annotations

import argparse
import importlib.util
import os
import re
import subprocess
import sys
import tomllib
from pathlib import Path
from typing import Any

from . import __version__ as ASTRIS_VERSION
from .app import Astris

MAIN_TEMPLATE = """import sys

from astris import Astris
from astris.lib import A, Body, Button, Div, H1, Header, Html, Main, P, Span

app = Astris()

# Inject Tailwind CSS via CDN
app.add_head_script("https://cdn.tailwindcss.com")


@app.page("/")
def home():
    return Html(
        **{"lang": "en", "class": "scroll-smooth"},
        children=[
            Body(
                **{
                    "class": "bg-slate-950 text-slate-100 min-h-screen flex flex-col font-sans antialiased selection:bg-indigo-500/30 selection:text-indigo-200"
                },
                children=[
                    # Background Effect
                    Div(
                        **{"class": "fixed inset-0 z-[-1]", "aria-hidden": "true"},
                        children=[
                            Div(
                                **{
                                    "class": "absolute top-0 -left-4 w-96 h-96 bg-indigo-500 rounded-full mix-blend-multiply filter blur-[128px] opacity-20 animate-pulse"
                                },
                                children=[]
                            ),
                            Div(
                                **{
                                    "class": "absolute top-0 -right-4 w-96 h-96 bg-purple-500 rounded-full mix-blend-multiply filter blur-[128px] opacity-20 animate-pulse animation-delay-2000"
                                },
                                children=[]
                            ),
                            Div(
                                **{
                                    "class": "absolute -bottom-8 left-20 w-96 h-96 bg-pink-500 rounded-full mix-blend-multiply filter blur-[128px] opacity-20 animate-pulse animation-delay-4000"
                                },
                                children=[]
                            ),
                        ]
                    ),
                    # Header Navigation
                    Header(
                        **{
                            "class": "w-full py-6 px-4 sm:px-6 lg:px-8 max-w-7xl mx-auto flex items-center justify-between"
                        },
                        children=[
                            Div(
                                **{
                                    "class": "flex items-center gap-2 group cursor-pointer"
                                },
                                children=[
                                    Div(
                                        **{
                                            "class": "w-8 h-8 rounded-lg bg-gradient-to-br from-indigo-500 to-purple-600 flex items-center justify-center transform group-hover:rotate-12 transition-transform duration-300 shadow-lg shadow-indigo-500/30"
                                        },
                                        children=[
                                            Span(
                                                **{
                                                    "class": "text-white font-bold text-lg"
                                                },
                                                children=["A"]
                                            )
                                        ]
                                    ),
                                    Span(
                                        **{
                                            "class": "text-xl font-bold tracking-tight bg-clip-text text-transparent bg-gradient-to-r from-white to-slate-400"
                                        },
                                        children=["Astris"]
                                    ),
                                ]
                            ),
                            Div(
                                **{
                                    "class": "flex items-center gap-6 text-sm font-medium"
                                },
                                children=[
                                    A(
                                        **{
                                            "href": "https://astris.readthedocs.io",
                                            "target": "_blank",
                                            "rel": "noopener noreferrer",
                                            "class": "text-slate-300 hover:text-white transition-colors",
                                        },
                                        children=["Documentation"]
                                    ),
                                    A(
                                        **{
                                            "href": "https://github.com/keoh/astris",
                                            "target": "_blank",
                                            "rel": "noopener noreferrer",
                                            "class": "text-slate-300 hover:text-white transition-colors",
                                        },
                                        children=["GitHub"]
                                    ),
                                ]
                            ),
                        ]
                    ),
                    # Main Content area (Hero Section)
                    Main(
                        **{
                            "class": "flex-grow flex flex-col items-center justify-center px-4 sm:px-6 lg:px-8 text-center mt-12 mb-24"
                        },
                        children=[
                            # Badge
                            A(
                                **{
                                    "href": "https://astris.readthedocs.io",
                                    "target": "_blank",
                                    "class": "inline-flex items-center gap-2 px-3 py-1 rounded-full bg-slate-800/50 border border-slate-700/50 text-sm font-medium text-slate-300 mb-8 backdrop-blur-sm hover:bg-slate-800 hover:border-slate-600 transition-all cursor-pointer group",
                                },
                                children=[
                                    Span(
                                        **{"class": "relative flex h-2 w-2"},
                                        children=[
                                            Span(
                                                **{
                                                    "class": "animate-ping absolute inline-flex h-full w-full rounded-full bg-indigo-400 opacity-75"
                                                },
                                                children=[]
                                            ),
                                            Span(
                                                **{
                                                    "class": "relative inline-flex rounded-full h-2 w-2 bg-indigo-500"
                                                },
                                                children=[]
                                            ),
                                        ]
                                    ),
                                    Span(children=["Astris Framework Ready"]),
                                    Span(
                                        **{
                                            "class": "text-indigo-400 group-hover:translate-x-1 transition-transform"
                                        },
                                        children=["→"]
                                    ),
                                ]
                            ),
                            # Title
                            H1(
                                **{
                                    "class": "text-5xl sm:text-6xl md:text-7xl font-extrabold tracking-tight mb-8 max-w-4xl mx-auto"
                                },
                                children=[
                                    Span(children=["Build static websites "]),
                                    Span(**{"class": "inline-block"}, children=[]),
                                    Span(
                                        **{
                                            "class": "text-transparent bg-clip-text bg-gradient-to-r from-indigo-400 via-purple-400 to-pink-400 hover:from-pink-400 hover:to-indigo-400 transition-all duration-1000"
                                        },
                                        children=["elegantly"]
                                    ),
                                ]
                            ),
                            # Description
                            P(
                                **{
                                    "class": "text-lg sm:text-xl text-slate-400 mb-10 max-w-2xl mx-auto leading-relaxed"
                                },
                                children=[
                                    "You are looking at your new Astris application. It's time to start building something amazing using component-style APIs in Python."
                                ]
                            ),
                            # Action Buttons
                            Div(
                                **{
                                    "class": "flex flex-col sm:flex-row gap-4 justify-center"
                                },
                                children=[
                                    A(
                                        **{
                                            "href": "https://astris.readthedocs.io/en/latest/quickstart/",
                                            "target": "_blank",
                                            "rel": "noopener noreferrer",
                                        },
                                        children=[
                                            Button(
                                                **{
                                                    "class": "px-8 py-4 w-full sm:w-auto rounded-xl bg-white text-slate-900 font-bold hover:bg-slate-100 hover:scale-[1.02] shadow-[0_0_40px_-10px_rgba(255,255,255,0.3)] transition-all duration-200 active:scale-95"
                                                },
                                                children=["Get Started"]
                                            )
                                        ]
                                    ),
                                    A(
                                        **{
                                            "href": "https://astris.readthedocs.io/en/latest/api/",
                                            "target": "_blank",
                                            "rel": "noopener noreferrer",
                                        },
                                        children=[
                                            Button(
                                                **{
                                                    "class": "px-8 py-4 w-full sm:w-auto rounded-xl bg-slate-800/80 text-white font-bold border border-slate-700 hover:bg-slate-800 hover:border-slate-500 hover:shadow-lg backdrop-blur-sm transition-all duration-200 active:scale-95"
                                                },
                                                children=["View API Reference"]
                                            )
                                        ]
                                    ),
                                ]
                            ),
                            # Code Snippet Preview (Glassmorphism card)
                            Div(
                                **{
                                    "class": "mt-16 w-full max-w-2xl mx-auto text-left relative group"
                                },
                                children=[
                                    # Decorative gradient border
                                    Div(
                                        **{
                                            "class": "absolute -inset-[1px] bg-gradient-to-r from-indigo-500/50 via-purple-500/50 to-pink-500/50 rounded-2xl blur-sm opacity-50 group-hover:opacity-100 transition duration-500"
                                        },
                                        children=[]
                                    ),
                                    Div(
                                        **{
                                            "class": "relative bg-slate-900/80 backdrop-blur-xl border border-slate-700/50 rounded-2xl overflow-hidden shadow-2xl"
                                        },
                                        children=[
                                            Div(
                                                **{
                                                    "class": "flex items-center px-4 py-3 bg-slate-800/50 border-b border-slate-700/50"
                                                },
                                                children=[
                                                    Div(
                                                        **{"class": "flex gap-2"},
                                                        children=[
                                                            Div(
                                                                **{
                                                                    "class": "w-3 h-3 rounded-full bg-red-500/80"
                                                                },
                                                                children=[]
                                                            ),
                                                            Div(
                                                                **{
                                                                    "class": "w-3 h-3 rounded-full bg-yellow-500/80"
                                                                },
                                                                children=[]
                                                            ),
                                                            Div(
                                                                **{
                                                                    "class": "w-3 h-3 rounded-full bg-green-500/80"
                                                                },
                                                                children=[]
                                                            ),
                                                        ]
                                                    ),
                                                    Span(
                                                        **{
                                                            "class": "ml-4 text-xs font-mono text-slate-400"
                                                        },
                                                        children=["main.py"]
                                                    ),
                                                ]
                                            ),
                                            Div(
                                                **{"class": "p-6 overflow-x-auto"},
                                                children=[
                                                    P(
                                                        **{
                                                            "class": "font-mono text-sm leading-relaxed"
                                                        },
                                                        children=[
                                                            Span(
                                                                **{
                                                                    "class": "text-purple-400"
                                                                },
                                                                children=["from "]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-slate-300"
                                                                },
                                                                children=["astris "]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-purple-400"
                                                                },
                                                                children=["import "]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-yellow-200"
                                                                },
                                                                children=["Astris"]
                                                            ),
                                                            Div(
                                                                **{"class": "h-2"},
                                                                children=[]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-slate-300"
                                                                },
                                                                children=["app = "]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-yellow-200"
                                                                },
                                                                children=["Astris"]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-slate-300"
                                                                },
                                                                children=["()"]
                                                            ),
                                                            Div(
                                                                **{"class": "h-2"},
                                                                children=[]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-yellow-400"
                                                                },
                                                                children=["@app.page"]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-slate-300"
                                                                },
                                                                children=["("]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-green-300"
                                                                },
                                                                children=['"/"']
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-slate-300"
                                                                },
                                                                children=[")"]
                                                            ),
                                                            Div(
                                                                **{"class": "h-0"},
                                                                children=[]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-purple-400"
                                                                },
                                                                children=["def "]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-blue-300"
                                                                },
                                                                children=["home"]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-slate-300"
                                                                },
                                                                children=["():"]
                                                            ),
                                                            Div(
                                                                **{"class": "h-0"},
                                                                children=[]
                                                            ),
                                                            Span(
                                                                **{
                                                                    "class": "text-slate-300 pl-4"
                                                                },
                                                                children=["..."]
                                                            ),
                                                        ]
                                                    )
                                                ]
                                            ),
                                        ]
                                    ),
                                ]
                            ),
                        ]
                    ),
                ]
            )
        ]
    )


if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1] == "build":
        app.build()
    else:
        app.run_dev(reload=True)

"""

README_TEMPLATE = """# {project_name}

Project generated with `astris new`.

## Install dependencies (uv)

```bash
uv sync
```

## Run in development

```bash
uv run python main.py
```

## Build static site

```bash
uv run astris build
```
"""

GITIGNORE_TEMPLATE = """__pycache__/
*.pyc
.venv/
dist/
"""

PYPROJECT_TEMPLATE = """[project]
name = "{project_name}"
version = "0.1.0"
requires-python = ">=3.11"
dependencies = [
    "astris=={astris_version}",
]

[tool.astris]

[tool.astris.build]
output_dir = "dist"
clean_urls = false

[tool.astris.deploy]
provider = "cloudflare"

[tool.astris.deploy.cloudflare]
project_name = "replace-with-your-cloudflare-project-name"

[tool.astris.env]
# EXAMPLE_API_URL = "https://api.example.com"
"""


class CliError(ValueError):
    """CLI usage error for project generation."""


def validate_project_name(project_name: str) -> None:
    """Validate that the provided project name is filesystem friendly."""
    pattern = r"^[A-Za-z][A-Za-z0-9_-]*$"
    if not re.match(pattern, project_name):
        raise CliError(
            "Invalid project name. Use letters, numbers, '_' or '-', and start with a letter."
        )


def create_project(project_name: str, base_path: Path | None = None) -> Path:
    """Create a minimal Astris project scaffold."""
    validate_project_name(project_name)

    root = (base_path or Path.cwd()) / project_name
    if root.exists():
        raise CliError(f"Target directory already exists: {root}")

    root.mkdir(parents=True, exist_ok=False)

    (root / "main.py").write_text(MAIN_TEMPLATE, encoding="utf-8")
    (root / "README.md").write_text(
        README_TEMPLATE.format(project_name=project_name), encoding="utf-8"
    )
    (root / ".gitignore").write_text(GITIGNORE_TEMPLATE, encoding="utf-8")
    (root / "pyproject.toml").write_text(
        PYPROJECT_TEMPLATE.format(
            project_name=project_name,
            astris_version=ASTRIS_VERSION,
        ),
        encoding="utf-8",
    )

    return root


def _load_app_from_file(file_path: str) -> Astris:
    target = Path(file_path).expanduser()
    if not target.is_absolute():
        target = Path.cwd() / target
    target = target.resolve()

    if not target.exists():
        raise CliError(f"Python file not found: {target}")
    if not target.is_file():
        raise CliError(f"Path is not a file: {target}")

    module_name = f"astris_build_target_{target.stem}"
    spec = importlib.util.spec_from_file_location(module_name, target)
    if spec is None or spec.loader is None:
        raise CliError(f"Could not load module from: {target}")

    module = importlib.util.module_from_spec(spec)
    try:
        spec.loader.exec_module(module)
    except Exception as error:  # pragma: no cover - exercised via CLI behavior
        raise CliError(f"Failed to execute {target}: {error}") from error

    app = getattr(module, "app", None)
    if app is None:
        raise CliError(f"No 'app' variable found in {target}")
    if not isinstance(app, Astris):
        raise CliError(f"'app' in {target} must be an Astris instance")

    return app


def _load_project_config(base_path: Path | None = None) -> dict[str, Any]:
    root = base_path or Path.cwd()
    pyproject_path = root / "pyproject.toml"
    if not pyproject_path.exists():
        return {}

    try:
        parsed = tomllib.loads(pyproject_path.read_text(encoding="utf-8"))
    except tomllib.TOMLDecodeError as error:
        raise CliError(f"Invalid TOML in {pyproject_path}: {error}") from error

    tool_section = parsed.get("tool", {})
    if not isinstance(tool_section, dict):
        raise CliError("Invalid pyproject.toml: [tool] must be a table")

    astris_section = tool_section.get("astris", {})
    if not isinstance(astris_section, dict):
        raise CliError("Invalid pyproject.toml: [tool.astris] must be a table")

    return astris_section


def _apply_config_env(config: dict[str, Any]) -> None:
    env_section = config.get("env", {})
    if not isinstance(env_section, dict):
        raise CliError("Invalid pyproject.toml: [tool.astris.env] must be a table")

    for key, value in env_section.items():
        os.environ[str(key)] = str(value)


def _extract_build_options(config: dict[str, Any]) -> tuple[str, bool]:
    build_section = config.get("build", {})
    if not isinstance(build_section, dict):
        raise CliError("Invalid pyproject.toml: [tool.astris.build] must be a table")

    output_dir = build_section.get("output_dir", "dist")
    if not isinstance(output_dir, str) or not output_dir.strip():
        raise CliError("Invalid pyproject.toml: build.output_dir must be a string")

    clean_urls = build_section.get("clean_urls", False)
    if not isinstance(clean_urls, bool):
        raise CliError("Invalid pyproject.toml: build.clean_urls must be true/false")

    return output_dir, clean_urls


def _extract_cloudflare_deploy_options(
    config: dict[str, Any],
) -> tuple[str, str | None]:
    deploy_section = config.get("deploy", {})
    if not isinstance(deploy_section, dict):
        raise CliError("Invalid pyproject.toml: [tool.astris.deploy] must be a table")

    provider = deploy_section.get("provider", "cloudflare")
    if provider != "cloudflare":
        raise CliError(
            f"Unsupported deploy provider: {provider}. Supported providers: cloudflare"
        )

    cloudflare_section = deploy_section.get("cloudflare", {})
    if not isinstance(cloudflare_section, dict):
        raise CliError(
            "Invalid pyproject.toml: [tool.astris.deploy.cloudflare] must be a table"
        )

    project_name = cloudflare_section.get("project_name")
    if not isinstance(project_name, str) or not project_name.strip():
        raise CliError(
            "Missing Cloudflare project name. Set tool.astris.deploy.cloudflare.project_name in pyproject.toml"
        )

    branch = cloudflare_section.get("branch")
    if branch is not None and (not isinstance(branch, str) or not branch.strip()):
        raise CliError(
            "Invalid pyproject.toml: deploy.cloudflare.branch must be a non-empty string"
        )

    return project_name, branch


def _run_cloudflare_pages_deploy(
    output_dir: str,
    project_name: str,
    branch: str | None = None,
) -> None:
    command = [
        "npx",
        "wrangler",
        "pages",
        "deploy",
        output_dir,
        "--project-name",
        project_name,
    ]
    if branch:
        command.extend(["--branch", branch])

    try:
        subprocess.run(command, check=True)
    except FileNotFoundError as error:
        raise CliError(
            "Could not run 'npx'. Install Node.js and ensure npx is available."
        ) from error
    except subprocess.CalledProcessError as error:
        raise CliError(f"Cloudflare deploy failed with exit code {error.returncode}") from error


def build_parser() -> argparse.ArgumentParser:
    """Build the top-level Astris CLI parser."""
    parser = argparse.ArgumentParser(
        prog="astris",
        description="Astris CLI",
    )
    subparsers = parser.add_subparsers(dest="command", required=True)

    new_parser = subparsers.add_parser("new", help="Create a new Astris project")
    new_parser.add_argument("project_name", help="Name of the project directory")

    build_parser = subparsers.add_parser("build", help="Build static site")
    build_parser.add_argument(
        "--file",
        default="main.py",
        help="Python app file to load (default: main.py)",
    )

    deploy_parser = subparsers.add_parser(
        "deploy", help="Build and deploy static site"
    )
    deploy_parser.add_argument(
        "--file",
        default="main.py",
        help="Python app file to load (default: main.py)",
    )

    return parser


def main(argv: list[str] | None = None) -> int:
    """CLI entrypoint."""
    parser = build_parser()
    args = parser.parse_args(argv)

    if args.command == "new":
        try:
            created = create_project(args.project_name)
        except CliError as error:
            print(f"Error: {error}", file=sys.stderr)
            return 2

        print(f"Created Astris project at {created}")
        print("Next steps:")
        print(f"  cd {args.project_name}")
        print("  uv run python main.py")
        return 0

    if args.command == "build":
        try:
            config = _load_project_config()
            _apply_config_env(config)
            output_dir, clean_urls = _extract_build_options(config)
            app = _load_app_from_file(args.file)
            app.build(output_dir=output_dir, clean_urls=clean_urls)
        except CliError as error:
            print(f"Error: {error}", file=sys.stderr)
            return 2

        print(f"Build complete: {output_dir}/")
        return 0

    if args.command == "deploy":
        try:
            config = _load_project_config()
            _apply_config_env(config)
            output_dir, clean_urls = _extract_build_options(config)
            project_name, branch = _extract_cloudflare_deploy_options(config)

            app = _load_app_from_file(args.file)
            app.build(output_dir=output_dir, clean_urls=clean_urls)
            _run_cloudflare_pages_deploy(
                output_dir=output_dir,
                project_name=project_name,
                branch=branch,
            )
        except CliError as error:
            print(f"Error: {error}", file=sys.stderr)
            return 2

        print(f"Deploy complete: cloudflare pages project '{project_name}'")
        return 0

    parser.print_help()
    return 1


if __name__ == "__main__":
    raise SystemExit(main())



import json
import re
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Callable, List, Sequence

from .component import Component

TemplateFactory = Callable[[Any], Component]


@dataclass(frozen=True)
class JsonCollectionEntry:
    slug: str
    route: str
    data: Any
    source: str


@dataclass(frozen=True)
class JsonCollection:
    name: str
    route_prefix: str
    api_prefix: str
    entries: Sequence[JsonCollectionEntry]

    def page_links(self) -> List[str]:
        return [entry.route for entry in self.entries]

    def to_api_payload(self) -> List[dict[str, Any]]:
        return [
            {
                "slug": entry.slug,
                "route": entry.route,
                "source": entry.source,
                "data": entry.data,
            }
            for entry in self.entries
        ]


def _slugify(value: str) -> str:
    candidate = re.sub(r"[^a-zA-Z0-9]+", "-", value.strip().lower()).strip("-")
    return candidate or "item"


def _normalize_prefix(prefix: str) -> str:
    if not prefix:
        raise ValueError("Prefix cannot be empty")

    normalized = prefix if prefix.startswith("/") else f"/{prefix}"
    if len(normalized) > 1:
        normalized = normalized.rstrip("/")
    return normalized


def _load_collection_entries(
    directory: str | Path,
    slug_field: str,
    route_prefix: str,
) -> List[JsonCollectionEntry]:
    source_dir = Path(directory)
    if not source_dir.exists():
        raise FileNotFoundError(f"Collection directory does not exist: {source_dir}")
    if not source_dir.is_dir():
        raise ValueError(f"Collection path must be a directory: {source_dir}")

    entries: List[JsonCollectionEntry] = []
    seen_slugs: set[str] = set()

    for json_file in sorted(source_dir.glob("*.json")):
        with open(json_file, "r", encoding="utf-8") as file:
            data = json.load(file)

        slug_value: str | None = None
        if isinstance(data, dict):
            value = data.get(slug_field)
            if isinstance(value, str) and value.strip():
                slug_value = value

        slug = _slugify(slug_value or json_file.stem)
        if slug in seen_slugs:
            raise ValueError(
                f"Duplicate slug '{slug}' in collection directory: {source_dir}"
            )
        seen_slugs.add(slug)

        route = f"{route_prefix}/{slug}"
        entries.append(
            JsonCollectionEntry(
                slug=slug,
                route=route,
                data=data,
                source=str(json_file.name),
            )
        )

    return entries


def register_json_collection(
    app: Any,
    *,
    name: str,
    directory: str | Path,
    template: TemplateFactory,
    api_prefix: str = "/api/collections",
    route_prefix: str | None = None,
    slug_field: str = "slug",
) -> JsonCollection:
    if not name.strip():
        raise ValueError("Collection name cannot be empty")

    normalized_name = name.strip()
    normalized_route_prefix = _normalize_prefix(route_prefix or f"/{normalized_name}")
    normalized_api_prefix = _normalize_prefix(api_prefix)

    entries = _load_collection_entries(directory, slug_field, normalized_route_prefix)

    for entry in entries:
        component = template(entry.data)
        if not isinstance(component, Component):
            raise TypeError("Collection template must return an Astris Component")
        app._register_component_route(entry.route, component)

    collection = JsonCollection(
        name=normalized_name,
        route_prefix=normalized_route_prefix,
        api_prefix=normalized_api_prefix,
        entries=entries,
    )
    app._register_collection_api(
        collection_name=normalized_name,
        entries=collection.to_api_payload(),
        api_prefix=normalized_api_prefix,
    )
    return collection



from typing import Optional, Sequence

from .component import Component
from .lib import Div


class Container(Div):
    """Convenience wrapper for a generic container block."""


class Column(Div):
    """Convenience wrapper that applies a vertical flex layout."""

    def __init__(
        self, children: Optional[Sequence[Component | str]] = None, **kwargs
    ) -> None:
        super().__init__(
            children, style="display: flex; flex-direction: column;", **kwargs
        )


class Row(Div):
    """Convenience wrapper that applies a horizontal flex layout."""

    def __init__(
        self, children: Optional[Sequence[Component | str]] = None, **kwargs
    ) -> None:
        super().__init__(
            children, style="display: flex; flex-direction: row;", **kwargs
        )



from astris.component import Component

from ..lib import Div, Img, H1, P


class CenteredHero(Div):
    """A simple centered hero section with a title and description."""

    def __init__(
        self,
        title: str,
        description: str,
        logo_img_url: str,
        actions: list[Component] = [],
    ):
        super().__init__(
            class_name="px-4 py-5 my-5 text-center",
            children=[
                Img(
                    src=logo_img_url,
                    alt=title,
                    width="72",
                    height="57",
                    class_name="d-block mx-auto mb-4",
                ),
                H1(
                    children=[title],
                    class_name="display-5 fw-bold text-body-emphasis",
                ),
                Div(
                    class_name="col-lg-6 mx-auto",
                    children=[
                        P(description, class_name="lead mb-4"),
                        Div(
                            class_name="d-grid gap-2 d-sm-flex justify-content-sm-center mb-5",
                            children=actions,
                        ),
                    ],
                ),
            ],
        )



import json
from pathlib import Path
from typing import Callable, cast

from fastapi import HTTPException
from fastapi.responses import JSONResponse

from astris import Astris, Text
from astris.content import register_json_collection
from astris.lib import Div


def _write_json(path: Path, payload: dict) -> None:
    path.write_text(json.dumps(payload), encoding="utf-8")


def _route_endpoint(app: Astris, path: str) -> Callable[..., JSONResponse]:
    for route in app._fastapi_app.routes:
        if getattr(route, "path", None) == path:
            endpoint = getattr(route, "endpoint", None)
            if callable(endpoint):
                return cast(Callable[..., JSONResponse], endpoint)
            break
    raise AssertionError(f"Route not found: {path}")


def test_register_json_collection_generates_detail_routes_and_build(
    tmp_path: Path,
) -> None:
    app = Astris()
    posts_dir = tmp_path / "posts"
    posts_dir.mkdir()

    _write_json(posts_dir / "hello-world.json", {"title": "Hello"})
    _write_json(posts_dir / "custom.json", {"slug": "custom-slug", "title": "Custom"})

    collection = register_json_collection(
        app,
        name="posts",
        directory=posts_dir,
        template=lambda entry: Div(children=[Text(entry["title"])]),
    )

    assert collection.page_links() == ["/posts/custom-slug", "/posts/hello-world"]

    output_dir = tmp_path / "site"
    app.build(str(output_dir))

    assert (output_dir / "posts/custom-slug.html").exists()
    assert (output_dir / "posts/hello-world.html").exists()


def test_register_json_collection_exposes_read_only_api(tmp_path: Path) -> None:
    app = Astris()
    posts_dir = tmp_path / "posts"
    posts_dir.mkdir()

    _write_json(posts_dir / "first.json", {"slug": "first", "title": "First"})

    register_json_collection(
        app,
        name="posts",
        directory=posts_dir,
        template=lambda entry: Div(children=[Text(entry["title"])]),
        api_prefix="/content-api",
    )

    list_endpoint = _route_endpoint(app, "/content-api/posts")
    detail_endpoint = _route_endpoint(app, "/content-api/posts/{slug}")

    list_response = list_endpoint()
    assert isinstance(list_response, JSONResponse)
    list_payload = json.loads(bytes(list_response.body).decode("utf-8"))
    assert len(list_payload) == 1
    assert list_payload[0]["slug"] == "first"
    assert list_payload[0]["route"] == "/posts/first"

    detail_response = detail_endpoint(slug="first")
    assert isinstance(detail_response, JSONResponse)
    detail_payload = json.loads(bytes(detail_response.body).decode("utf-8"))
    assert detail_payload["data"]["title"] == "First"

    try:
        detail_endpoint(slug="missing")
    except HTTPException as exc:
        assert exc.status_code == 404
    else:
        raise AssertionError("Expected HTTPException(404) for missing collection slug")


def test_register_json_collection_template_must_return_component(
    tmp_path: Path,
) -> None:
    app = Astris()
    posts_dir = tmp_path / "posts"
    posts_dir.mkdir()
    _write_json(posts_dir / "first.json", {"title": "First"})

    try:
        register_json_collection(
            app,
            name="posts",
            directory=posts_dir,
            template=lambda entry: entry,
        )
    except TypeError as exc:
        assert "Component" in str(exc)
    else:
        raise AssertionError(
            "register_json_collection should fail for invalid template"
        )



from pathlib import Path

from astris import __version__, cli


def test_create_project_generates_expected_files(tmp_path: Path) -> None:
    project_path = cli.create_project("demo_site", base_path=tmp_path)

    assert project_path == tmp_path / "demo_site"
    assert (project_path / "main.py").exists()
    assert (project_path / "README.md").exists()
    assert (project_path / ".gitignore").exists()
    assert not (project_path / "requirements.txt").exists()
    assert (project_path / "pyproject.toml").exists()

    main_content = (project_path / "main.py").read_text(encoding="utf-8")
    pyproject_content = (project_path / "pyproject.toml").read_text(encoding="utf-8")
    assert "app = Astris()" in main_content
    assert '@app.page("/")' in main_content
    assert f'"astris=={__version__}"' in pyproject_content


def test_create_project_fails_if_target_exists(tmp_path: Path) -> None:
    existing = tmp_path / "demo_site"
    existing.mkdir()

    try:
        cli.create_project("demo_site", base_path=tmp_path)
        assert False, "Expected CliError"
    except cli.CliError as error:
        assert "already exists" in str(error)


def test_create_project_rejects_invalid_name(tmp_path: Path) -> None:
    try:
        cli.create_project("123-invalid", base_path=tmp_path)
        assert False, "Expected CliError"
    except cli.CliError as error:
        assert "Invalid project name" in str(error)


def test_main_new_command_returns_success(tmp_path: Path, monkeypatch) -> None:
    monkeypatch.chdir(tmp_path)

    exit_code = cli.main(["new", "hello_app"])

    assert exit_code == 0
    assert (tmp_path / "hello_app" / "main.py").exists()


def test_main_build_command_uses_main_py_by_default(
    tmp_path: Path, monkeypatch
) -> None:
    monkeypatch.chdir(tmp_path)
    (tmp_path / "main.py").write_text(
        """
from astris import Astris
from astris.lib import Body, Html

app = Astris()


@app.page("/")
def home():
    return Html(children=[Body(children=["hello"])])
""".strip(),
        encoding="utf-8",
    )

    exit_code = cli.main(["build"])

    assert exit_code == 0
    assert (tmp_path / "dist" / "index.html").exists()


def test_main_build_command_supports_custom_file(tmp_path: Path, monkeypatch) -> None:
    monkeypatch.chdir(tmp_path)
    (tmp_path / "example.py").write_text(
        """
from astris import Astris
from astris.lib import Body, Html

app = Astris()


@app.page("/")
def home():
    return Html(children=[Body(children=["hello from example"])])
""".strip(),
        encoding="utf-8",
    )

    exit_code = cli.main(["build", "--file", "example.py"])

    assert exit_code == 0
    output = (tmp_path / "dist" / "index.html").read_text(encoding="utf-8")
    assert "hello from example" in output


def test_main_build_command_uses_pyproject_build_settings(
    tmp_path: Path, monkeypatch
) -> None:
    monkeypatch.chdir(tmp_path)
    (tmp_path / "main.py").write_text(
        """
from astris import Astris
from astris.lib import A, Body, Html

app = Astris()


@app.page("/")
def home():
    return Html(children=[Body(children=[A(href="/about", children=["About"])])])


@app.page("/about")
def about():
    return Html(children=[Body(children=["about page"])])
""".strip(),
        encoding="utf-8",
    )
    (tmp_path / "pyproject.toml").write_text(
        """
[tool.astris.build]
output_dir = "public"
clean_urls = true
""".strip(),
        encoding="utf-8",
    )

    exit_code = cli.main(["build"])

    assert exit_code == 0
    assert (tmp_path / "public" / "index.html").exists()
    assert (tmp_path / "public" / "about" / "index.html").exists()


def test_main_build_command_applies_env_from_pyproject(
    tmp_path: Path, monkeypatch
) -> None:
    monkeypatch.chdir(tmp_path)
    (tmp_path / "main.py").write_text(
        """
import os

from astris import Astris
from astris.lib import Body, Html

app = Astris()


@app.page("/")
def home():
    return Html(children=[Body(children=[os.environ.get("ASTRIS_TEST_ENV", "missing")])])
""".strip(),
        encoding="utf-8",
    )
    (tmp_path / "pyproject.toml").write_text(
        """
[tool.astris.env]
ASTRIS_TEST_ENV = "available"
""".strip(),
        encoding="utf-8",
    )

    exit_code = cli.main(["build"])

    assert exit_code == 0
    output = (tmp_path / "dist" / "index.html").read_text(encoding="utf-8")
    assert "available" in output


def test_main_deploy_runs_cloudflare_pages_command(tmp_path: Path, monkeypatch) -> None:
    monkeypatch.chdir(tmp_path)
    (tmp_path / "main.py").write_text(
        """
from astris import Astris
from astris.lib import Body, Html

app = Astris()


@app.page("/")
def home():
    return Html(children=[Body(children=["hello"])])
""".strip(),
        encoding="utf-8",
    )
    (tmp_path / "pyproject.toml").write_text(
        """
[tool.astris.build]
output_dir = "dist"
clean_urls = true

[tool.astris.deploy]
provider = "cloudflare"

[tool.astris.deploy.cloudflare]
project_name = "my-pages-project"
branch = "main"
""".strip(),
        encoding="utf-8",
    )

    captured: list[list[str]] = []

    def fake_run(command: list[str], check: bool) -> None:
        assert check is True
        captured.append(command)

    monkeypatch.setattr(cli.subprocess, "run", fake_run)

    exit_code = cli.main(["deploy"])

    assert exit_code == 0
    assert (tmp_path / "dist" / "index.html").exists()
    assert captured == [
        [
            "npx",
            "wrangler",
            "pages",
            "deploy",
            "dist",
            "--project-name",
            "my-pages-project",
            "--branch",
            "main",
        ]
    ]


def test_main_build_command_fails_if_file_missing(tmp_path: Path, monkeypatch) -> None:
    monkeypatch.chdir(tmp_path)

    exit_code = cli.main(["build"])

    assert exit_code == 2


def test_main_build_command_fails_if_app_not_found(tmp_path: Path, monkeypatch) -> None:
    monkeypatch.chdir(tmp_path)
    (tmp_path / "main.py").write_text("x = 1", encoding="utf-8")

    exit_code = cli.main(["build"])

    assert exit_code == 2


def test_main_build_command_fails_if_app_type_is_invalid(
    tmp_path: Path, monkeypatch
) -> None:
    monkeypatch.chdir(tmp_path)
    (tmp_path / "main.py").write_text("app = object()", encoding="utf-8")

    exit_code = cli.main(["build"])

    assert exit_code == 2



import inspect

import astris.lib as lib
from astris.component import Element
from astris.layout import Column, Container, Row
from astris.lib import A, Br, Head, Html, Img, Title


def test_html_wrappers_render_expected_tags() -> None:
    page = Html(
        children=[
            Head(children=[Title(children=["Demo"])]),
            Container(children=[A(href="/", children=["Inicio"])]),
        ]
    )
    html = page.render()

    assert "<html>" in html
    assert "<head>" in html
    assert "<title>Demo</title>" in html
    assert '<a href="/">Inicio</a>' in html


def test_void_wrappers_render_without_closing_tag() -> None:
    image = Img(src="/logo.png", alt="logo")
    line_break = Br()

    assert image.render() == '<img src="/logo.png" alt="logo">'
    assert line_break.render() == "<br>"


def test_column_and_row_include_default_flex_style() -> None:
    column = Column(children=["x"])
    row = Row(children=["y"])

    assert "flex-direction: column" in column.render()
    assert "flex-direction: row" in row.render()


def test_all_element_classes_in_lib_include_docstrings() -> None:
    element_classes = [
        value
        for _, value in vars(lib).items()
        if inspect.isclass(value)
        and issubclass(value, Element)
        and value is not Element
    ]

    assert element_classes
    for cls in element_classes:
        assert cls.__doc__ is not None
        assert cls.__doc__.strip()


def test_lib_includes_expected_core_html_wrappers() -> None:
    expected = {
        "Html",
        "Head",
        "Body",
        "Main",
        "Section",
        "Header",
        "Footer",
        "Nav",
        "Form",
        "Input",
        "Button",
        "Select",
        "Textarea",
        "Table",
        "Video",
        "Audio",
        "Canvas",
        "Template",
    }

    available = {name for name, value in vars(lib).items() if inspect.isclass(value)}
    assert expected.issubset(available)



from .component import Element


class A(Element):
    """Creates a hyperlink to another page or location."""

    tag = "a"


class Abbr(Element):
    """Represents an abbreviation or acronym."""

    tag = "abbr"


class Address(Element):
    """Provides contact information for a person or organization."""

    tag = "address"


class Area(Element):
    """Defines a clickable area in an image map."""

    tag = "area"


class Article(Element):
    """Represents a self-contained piece of content."""

    tag = "article"


class Aside(Element):
    """Represents content related to surrounding content."""

    tag = "aside"


class Audio(Element):
    """Embeds sound content."""

    tag = "audio"


class B(Element):
    """Draws attention to text without extra semantic importance."""

    tag = "b"


class Base(Element):
    """Specifies the base URL used for relative URLs in the document."""

    tag = "base"


class Bdi(Element):
    """Isolates text direction from surrounding text."""

    tag = "bdi"


class Bdo(Element):
    """Overrides the current text direction."""

    tag = "bdo"


class Blockquote(Element):
    """Represents a section quoted from another source."""

    tag = "blockquote"


class Body(Element):
    """Contains the visible content of the document."""

    tag = "body"


class Br(Element):
    """Inserts a line break."""

    tag = "br"


class Button(Element):
    """Represents a clickable button control."""

    tag = "button"


class Canvas(Element):
    """Provides a bitmap drawing surface scripted with JavaScript."""

    tag = "canvas"


class Caption(Element):
    """Represents the caption of a table."""

    tag = "caption"


class Cite(Element):
    """Represents the title of a referenced creative work."""

    tag = "cite"


class Code(Element):
    """Represents a fragment of computer code."""

    tag = "code"


class Col(Element):
    """Represents a column within a table column group."""

    tag = "col"


class Colgroup(Element):
    """Groups columns in a table for shared formatting."""

    tag = "colgroup"


class Data(Element):
    """Associates content with a machine-readable value."""

    tag = "data"


class Datalist(Element):
    """Provides predefined options for input controls."""

    tag = "datalist"


class Dd(Element):
    """Represents a description entry in a description list."""

    tag = "dd"


class Del(Element):
    """Represents deleted content."""

    tag = "del"


class Details(Element):
    """Represents a disclosure widget for expandable details."""

    tag = "details"


class Dfn(Element):
    """Represents the defining instance of a term."""

    tag = "dfn"


class Dialog(Element):
    """Represents a dialog box or interactive component."""

    tag = "dialog"


class Div(Element):
    """Represents a generic block container."""

    tag = "div"


class Dl(Element):
    """Represents a description list."""

    tag = "dl"


class Dt(Element):
    """Represents a term in a description list."""

    tag = "dt"


class Em(Element):
    """Represents stress emphasis."""

    tag = "em"


class Embed(Element):
    """Embeds external content at the insertion point."""

    tag = "embed"


class Fieldset(Element):
    """Groups related form controls."""

    tag = "fieldset"


class Figcaption(Element):
    """Represents a caption for figure content."""

    tag = "figcaption"


class Figure(Element):
    """Represents self-contained content with optional caption."""

    tag = "figure"


class Footer(Element):
    """Represents footer content for a page or section."""

    tag = "footer"


class Form(Element):
    """Represents an interactive form."""

    tag = "form"


class H1(Element):
    """Represents a top-level heading."""

    tag = "h1"


class H2(Element):
    """Represents a second-level heading."""

    tag = "h2"


class H3(Element):
    """Represents a third-level heading."""

    tag = "h3"


class H4(Element):
    """Represents a fourth-level heading."""

    tag = "h4"


class H5(Element):
    """Represents a fifth-level heading."""

    tag = "h5"


class H6(Element):
    """Represents a sixth-level heading."""

    tag = "h6"


class Head(Element):
    """Contains metadata for the document."""

    tag = "head"


class Header(Element):
    """Represents introductory content for a page or section."""

    tag = "header"


class Hr(Element):
    """Represents a thematic break between paragraphs."""

    tag = "hr"


class Html(Element):
    """Represents the root element of an HTML document."""

    tag = "html"


class I(Element):  # noqa: E742
    """Represents text in an alternate voice or mood."""

    tag = "i"


class Iframe(Element):
    """Embeds another HTML document."""

    tag = "iframe"


class Img(Element):
    """Embeds an image."""

    tag = "img"


class Input(Element):
    """Represents an input control."""

    tag = "input"


class Ins(Element):
    """Represents inserted content."""

    tag = "ins"


class Kbd(Element):
    """Represents user input from a keyboard or similar device."""

    tag = "kbd"


class Label(Element):
    """Represents a caption for a form control."""

    tag = "label"


class Legend(Element):
    """Represents a caption for a fieldset."""

    tag = "legend"


class Use(Element):
    """Represents a <use> element in SVG."""

    tag = "use"


class Li(Element):
    """Represents a list item."""

    tag = "li"


class Link(Element):
    """Links an external resource, such as a stylesheet."""

    tag = "link"


class Main(Element):
    """Represents the dominant content of the document body."""

    tag = "main"


class Map(Element):
    """Defines an image map with clickable regions."""

    tag = "map"


class Mark(Element):
    """Represents highlighted text for reference."""

    tag = "mark"


class Menu(Element):
    """Represents a list of commands or options."""

    tag = "menu"


class Meta(Element):
    """Represents metadata not covered by other head elements."""

    tag = "meta"


class Meter(Element):
    """Represents a scalar measurement within a known range."""

    tag = "meter"


class Nav(Element):
    """Represents a section containing navigation links."""

    tag = "nav"


class Noscript(Element):
    """Represents fallback content when scripts are unavailable."""

    tag = "noscript"


class Object(Element):
    """Represents an external resource, such as media."""

    tag = "object"


class Ol(Element):
    """Represents an ordered list."""

    tag = "ol"


class Optgroup(Element):
    """Groups related options in a select control."""

    tag = "optgroup"


class Option(Element):
    """Represents a selectable option in a form control."""

    tag = "option"


class Output(Element):
    """Represents the result of a calculation or user action."""

    tag = "output"


class P(Element):
    """Represents a paragraph."""

    tag = "p"


class Param(Element):
    """Defines parameters for an object element."""

    tag = "param"


class Picture(Element):
    """Provides multiple image resources for responsive rendering."""

    tag = "picture"


class Pre(Element):
    """Represents preformatted text."""

    tag = "pre"


class Progress(Element):
    """Represents progress completion for a task."""

    tag = "progress"


class Q(Element):
    """Represents a short inline quotation."""

    tag = "q"


class Rp(Element):
    """Provides fallback parentheses for ruby annotations."""

    tag = "rp"


class Rt(Element):
    """Represents ruby annotation text."""

    tag = "rt"


class Ruby(Element):
    """Represents ruby annotations for East Asian typography."""

    tag = "ruby"


class S(Element):
    """Represents content that is no longer accurate or relevant."""

    tag = "s"


class Samp(Element):
    """Represents sample output from a program."""

    tag = "samp"


class Script(Element):
    """Embeds executable script content."""

    tag = "script"


class Section(Element):
    """Represents a standalone section of content."""

    tag = "section"


class Select(Element):
    """Represents a select control."""

    tag = "select"


class Slot(Element):
    """Defines an insertion point in a web component template."""

    tag = "slot"


class Small(Element):
    """Represents side comments or small print."""

    tag = "small"


class Source(Element):
    """Specifies media resources for media or picture elements."""

    tag = "source"


class Span(Element):
    """Represents a generic inline container."""

    tag = "span"


class Strong(Element):
    """Represents strong importance."""

    tag = "strong"


class Style(Element):
    """Embeds CSS style rules."""

    tag = "style"


class Sub(Element):
    """Represents subscript text."""

    tag = "sub"


class Summary(Element):
    """Represents a summary for a details element."""

    tag = "summary"


class Sup(Element):
    """Represents superscript text."""

    tag = "sup"


class Svg(Element):
    """Represents an SVG drawing container."""

    tag = "svg"


class Table(Element):
    """Represents tabular data."""

    tag = "table"


class Tbody(Element):
    """Groups body rows in a table."""

    tag = "tbody"


class Td(Element):
    """Represents a data cell in a table."""

    tag = "td"


class Template(Element):
    """Holds markup fragments that are not rendered immediately."""

    tag = "template"


class Textarea(Element):
    """Represents a multi-line plain-text editing control."""

    tag = "textarea"


class Tfoot(Element):
    """Groups footer rows in a table."""

    tag = "tfoot"


class Th(Element):
    """Represents a header cell in a table."""

    tag = "th"


class Thead(Element):
    """Groups header rows in a table."""

    tag = "thead"


class Time(Element):
    """Represents a specific date or time."""

    tag = "time"


class Title(Element):
    """Represents the document title."""

    tag = "title"


class Tr(Element):
    """Represents a row in a table."""

    tag = "tr"


class Track(Element):
    """Specifies timed text tracks for media elements."""

    tag = "track"


class U(Element):
    """Represents non-textual annotation styling."""

    tag = "u"


class Ul(Element):
    """Represents an unordered list."""

    tag = "ul"


class Var(Element):
    """Represents a variable in code or math expressions."""

    tag = "var"


class Video(Element):
    """Embeds video content."""

    tag = "video"


class Wbr(Element):
    """Represents a line-break opportunity."""

    tag = "wbr"



import os
import re
import sys
from pathlib import Path
from typing import Any, Dict, List
from urllib.parse import urlsplit, urlunsplit

import uvicorn
from fastapi import HTTPException
from fastapi import FastAPI
from fastapi.responses import JSONResponse
from fastapi.responses import HTMLResponse

from .component import Component


class Astris:
    def __init__(self):
        self.routes: Dict[str, Component] = {}
        self._collection_entries: Dict[str, List[Dict[str, Any]]] = {}
        self._collection_api_prefixes: Dict[str, str] = {}
        self._head_links: List[Dict[str, str]] = []
        self._head_scripts: List[Dict[str, str]] = []
        self._fastapi_app = FastAPI()

    def _normalize_route_path(self, path: str) -> str:
        if not path:
            raise ValueError("Route path cannot be empty")

        normalized = path if path.startswith("/") else f"/{path}"
        if len(normalized) > 1:
            normalized = normalized.rstrip("/")
        return normalized

    def _register_component_route(self, path: str, component: Component) -> str:
        normalized_path = self._normalize_route_path(path)
        if normalized_path in self.routes:
            raise ValueError(f"Route already registered: {normalized_path}")

        self.routes[normalized_path] = component
        component_tree = component

        def serve_page():
            return f"<!DOCTYPE html>{self._render_page_html(component_tree)}"

        self._fastapi_app.add_api_route(
            normalized_path,
            serve_page,
            methods=["GET"],
            response_class=HTMLResponse,
        )

        return normalized_path

    def _register_collection_api(
        self,
        collection_name: str,
        entries: List[Dict[str, Any]],
        api_prefix: str,
    ) -> None:
        normalized_prefix = self._normalize_route_path(api_prefix)

        if collection_name in self._collection_api_prefixes:
            previous = self._collection_api_prefixes[collection_name]
            if previous != normalized_prefix:
                raise ValueError(
                    "Collection already registered with a different API prefix: "
                    f"{collection_name}"
                )
            raise ValueError(f"Collection already registered: {collection_name}")

        self._collection_entries[collection_name] = entries
        self._collection_api_prefixes[collection_name] = normalized_prefix

        base_path = f"{normalized_prefix}/{collection_name}"
        collection_entries = self._collection_entries[collection_name]

        def list_entries():
            return JSONResponse(content=collection_entries)

        def get_entry(slug: str):
            for item in collection_entries:
                if item.get("slug") == slug:
                    return JSONResponse(content=item)
            raise HTTPException(status_code=404, detail="Collection entry not found")

        self._fastapi_app.add_api_route(
            base_path,
            list_entries,
            methods=["GET"],
            response_class=JSONResponse,
        )
        self._fastapi_app.add_api_route(
            f"{base_path}/{{slug}}",
            get_entry,
            methods=["GET"],
            response_class=JSONResponse,
        )

    def add_head_link(self, href: str, rel: str = "stylesheet", **attributes) -> None:
        """Register a link tag to be injected in the document head."""
        link_attrs = {"rel": rel, "href": href, **attributes}
        self._head_links.append({key: str(value) for key, value in link_attrs.items()})

    def add_head_script(self, src: str, **attributes) -> None:
        """Register a script tag to be injected in the document head."""
        script_attrs = {"src": src, **attributes}
        self._head_scripts.append(
            {key: str(value) for key, value in script_attrs.items()}
        )

    def _render_tag_attributes(self, attributes: Dict[str, str]) -> str:
        return " ".join([f'{key}="{value}"' for key, value in attributes.items()])

    def _render_head_assets(self) -> str:
        parts: List[str] = []
        for link_attrs in self._head_links:
            attrs = self._render_tag_attributes(link_attrs)
            parts.append(f"<link {attrs}>")

        for script_attrs in self._head_scripts:
            attrs = self._render_tag_attributes(script_attrs)
            parts.append(f"<script {attrs}></script>")

        return "".join(parts)

    def _inject_head_assets(self, html: str) -> str:
        head_assets = self._render_head_assets()
        if not head_assets:
            return html

        head_close_match = re.search(r"</head>", html, re.IGNORECASE)
        if head_close_match:
            insert_at = head_close_match.start()
            return f"{html[:insert_at]}{head_assets}{html[insert_at:]}"

        html_open_match = re.search(r"<html[^>]*>", html, re.IGNORECASE)
        if html_open_match:
            insert_at = html_open_match.end()
            return f"{html[:insert_at]}<head>{head_assets}</head>{html[insert_at:]}"

        return f"<head>{head_assets}</head>{html}"

    def _render_page_html(self, component: Component) -> str:
        return self._inject_head_assets(component.render())

    def page(self, path: str):
        """Decorator to register a page (route)."""

        def decorator(func):
            component_tree = func()
            self._register_component_route(path, component_tree)

            return func

        return decorator

    def _infer_import_string(self) -> str | None:
        main_module = sys.modules.get("__main__")
        main_file = getattr(main_module, "__file__", None)
        if not main_file:
            return None

        module_name = Path(main_file).stem
        return f"{module_name}:app._fastapi_app"

    def run_dev(self, port=8000, reload=True):
        """Start the development server."""
        print(f"🚀 Dev server running at http://localhost:{port}")

        if reload:
            import_string = self._infer_import_string()
            if import_string:
                uvicorn.run(
                    import_string,
                    host="0.0.0.0",
                    port=port,
                    reload=True,
                    reload_dirs=[os.getcwd()],
                )
                return

            print("⚠️ Could not infer the main module, starting without hot reload.")

        uvicorn.run(self._fastapi_app, host="0.0.0.0", port=port)

    def _route_to_filename(self, route: str, clean_urls: bool = False) -> str:
        normalized = self._normalize_route_path(route)
        if normalized == "/":
            return "index.html"
        if clean_urls:
            return f"{normalized.strip('/')}/index.html"
        return f"{normalized.strip('/')}.html"

    def _resolve_route(self, path: str) -> str | None:
        if not path:
            return None

        normalized_path = self._normalize_route_path(path)

        if normalized_path in self.routes:
            return normalized_path

        without_slash = normalized_path.rstrip("/")
        if without_slash and without_slash in self.routes:
            return without_slash

        with_slash = f"{without_slash}/"
        if with_slash in self.routes:
            return with_slash

        return None

    def _rewrite_static_links(
        self, current_route: str, html: str, clean_urls: bool = False
    ) -> str:
        current_file = self._route_to_filename(current_route, clean_urls=clean_urls)
        current_dir = os.path.dirname(current_file) or "."

        def replace_href(match: re.Match[str]) -> str:
            quote = match.group("quote")
            href_value = match.group("href")

            split = urlsplit(href_value)
            if split.scheme or split.netloc or split.path.startswith("//"):
                return match.group(0)

            if split.path and not split.path.startswith("/"):
                return match.group(0)

            resolved_route = self._resolve_route(split.path)
            if not resolved_route:
                return match.group(0)

            if clean_urls:
                target_path = "/" if resolved_route == "/" else resolved_route
                rebuilt_href = urlunsplit(
                    ("", "", target_path, split.query, split.fragment)
                )
            else:
                target_file = self._route_to_filename(resolved_route)
                relative_target = os.path.relpath(target_file, start=current_dir).replace(
                    os.sep, "/"
                )
                rebuilt_href = urlunsplit(
                    ("", "", relative_target, split.query, split.fragment)
                )

            return f"href={quote}{rebuilt_href}{quote}"

        return re.sub(
            r'href=(?P<quote>["\'])(?P<href>.*?)(?P=quote)',
            replace_href,
            html,
        )

    def build(self, output_dir="dist", clean_urls: bool = False):
        """Generate static HTML files."""
        print(f"📦 Building site into ./{output_dir}...")

        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        for path, component in self.routes.items():
            filename = self._route_to_filename(path, clean_urls=clean_urls)
            filepath = os.path.join(output_dir, filename)
            os.makedirs(os.path.dirname(filepath), exist_ok=True)

            rendered_html = self._rewrite_static_links(
                path,
                self._render_page_html(component),
                clean_urls=clean_urls,
            )

            with open(filepath, "w", encoding="utf-8") as f:
                f.write("<!DOCTYPE html>\n")
                f.write(rendered_html)

            print(f"  ✅ Generated: {filepath}")

        print("✨ Build completed successfully.")
